Attribute VB_Name = "modFileBMP"
Function IsBMP(Filename As String) As Boolean
    Dim bmfh As BITMAPFILEHEADER         'Stores file header
    Open Filename For Binary Access Read As #1
        'Get the file header
        Get #1, , bmfh
    Close #1
    If bmfh.bfType = 19778 Then IsBMP = True Else IsBMP = False                 '0x4d42 or 'BM'
End Function
Sub LoadBMP(Filename As String, ByRef pImage As ImageFile)
    
    Dim bmfh As BITMAPFILEHEADER         'Stores file header
    Dim bmih As BITMAPINFOHEADER         'Stores info header
    Dim bmRGBQuad(0 To 255) As RGBQUAD   'Stores colour palette
    Dim bmData() As Byte                 'Stores pixel data
    Dim offset1 As Long, Offset2 As Long
    
    Open Filename For Binary Access Read As #1
        'Get the file header
        Get #1, , bmfh
        'Get the Bitmap Info (i.e size/colour depth etc)
        Get #1, , bmih
        'Depending on the colour depth, we will need
        'to resize our palette array to hold the
        'required amount of colours.
        Select Case bmih.biBitCount
            Case 8
                'If the bitmap is 8bpp then it uses a palette
                'so read it in, otherwise skip it.
                Get #1, , bmRGBQuad
            Case 24
                'Our favourite format =)
            Case Else
                Erase bmData
                MsgBox "Image format unsupported", vbCritical, "Error"
                Close #1
                Exit Sub
        End Select
        'Dimension the data array to store our image.
        ReDim bmData(1 To bmih.biSizeImage)
        'Get the image data
        Get #1, , bmData
    Close #1
    
    'Copy this data into our custom image object (which was passed).
    'Remember that Bitmaps are stored from bottom to top, so we need
    'to switch the order as we load (hence starting from the bottom line)
    pImage.ImageBPP = bmih.biBitCount
    pImage.ImageWidth = bmih.biWidth
    pImage.ImageHeight = bmih.biHeight
    If pImage.ImageBPP = 8 Then
        ReDim pImage.ImagePalette(0 To 255)
        CopyMemory pImage.ImagePalette(0), bmRGBQuad(0), 256 * 4
        ReDim pImage.ImageData(1 To (pImage.ImageWidth * pImage.ImageHeight))
        offset1 = UBound(pImage.ImageData): offset1 = offset1 - (pImage.ImageWidth - 1)
        Offset2 = 1
        For i = pImage.ImageHeight - 1 To 0 Step -1
            CopyMemory pImage.ImageData(offset1), bmData(Offset2), pImage.ImageWidth
            offset1 = offset1 - (pImage.ImageWidth)
            Offset2 = Offset2 + (pImage.ImageWidth)
        Next i
    Else
        ReDim pImage.ImageData(1 To ((pImage.ImageWidth * pImage.ImageHeight) * 3))
        Erase pImage.ImagePalette
        offset1 = UBound(pImage.ImageData): offset1 = offset1 - ((pImage.ImageWidth * 3) - 1)
        Offset2 = 1
        For i = pImage.ImageHeight - 1 To 0 Step -1
            CopyMemory pImage.ImageData(offset1), bmData(Offset2), (pImage.ImageWidth * 3)
            offset1 = offset1 - (pImage.ImageWidth * 3)
            Offset2 = Offset2 + (pImage.ImageWidth * 3)
        Next i
    End If
    Erase bmData
End Sub

Sub SaveBMP(Filename As String, ByRef pImage As ImageFile)
    
    Dim bmfh As BITMAPFILEHEADER         'Stores file header
    Dim bmih As BITMAPINFOHEADER         'Stores info header
    Dim bmRGBQuad(0 To 255) As RGBQUAD   'Stores colour palette
    Dim bmData() As Byte                 'Stores pixel data
    
    bmih.biSize = 40
    bmih.biWidth = pImage.ImageWidth
    bmih.biHeight = pImage.ImageHeight
    bmih.biPlanes = 1
    bmih.biBitCount = pImage.ImageBPP
    bmih.biCompression = 0
    bmih.biXPelsPerMeter = 2835         ' 72 dpi
    bmih.biYPelsPerMeter = 2835         ' 72 dpi
    bmih.biClrUsed = IIf(pImage.ImageBPP = 8, 256, 0)
    bmih.biClrImportant = 0
    bmih.biSizeImage = (pImage.ImageWidth * pImage.ImageHeight) * (pImage.ImageBPP / 8)
    
    bmfh.bfType = 19778                 '0x4d42 or 'BM'
    bmfh.bfReserved1 = 0
    bmfh.bfReserved2 = 0
    bmfh.bfSize = 54 + bmih.biSizeImage + bmih.biClrUsed * 4
    bmfh.bfOffBits = 54 + bmih.biClrUsed * 4
    
    If Dir(Filename) <> "" Then Kill Filename
    Open Filename For Binary Access Write As #1
        'Get the file header
        Put #1, , bmfh
        'Get the Bitmap Info (i.e size/colour depth etc)
        Put #1, , bmih
        'Depending on the colour depth, we will need
        'to resize our palette array to hold the
        'required amount of colours.
        Select Case bmih.biBitCount
            Case 8
                'If the bitmap is 8bpp then it uses a palette
                'so read it in, otherwise skip it.
                CopyMemory bmRGBQuad(0), pImage.ImagePalette(0), 256 * 4
                Put #1, , bmRGBQuad
            Case 24
                'Our favourite format =)
            Case Else
                Erase bmData
                MsgBox "Image format unsupported", vbCritical, "Error"
                Exit Sub
        End Select
        'Dimension the data array to store our image.
        ReDim bmData(1 To bmih.biSizeImage)
        CopyMemory bmData(1), pImage.ImageData(1), UBound(pImage.ImageData)
        'Get the image data
        Put #1, , bmData
    Close #1
    Erase bmData
End Sub

